CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
150 lines
4.8 KiB
C++
150 lines
4.8 KiB
C++
/* ************************************************************************
|
|
* Copyright (C) 2026 Advanced Micro Devices, Inc. All rights Reserved.
|
|
*
|
|
* Permission is hereby granted, free of charge, to any person obtaining a copy
|
|
* of this software and associated documentation files (the "Software"), to deal
|
|
* in the Software without restriction, including without limitation the rights
|
|
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
|
|
* copies of the Software, and to permit persons to whom the Software is
|
|
* furnished to do so, subject to the following conditions:
|
|
*
|
|
* The above copyright notice and this permission notice shall be included in
|
|
* all copies or substantial portions of the Software.
|
|
*
|
|
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
|
|
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
|
|
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
|
|
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
|
|
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
|
|
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
|
|
* THE SOFTWARE.
|
|
*
|
|
* ************************************************************************ */
|
|
|
|
#pragma once
|
|
#ifndef HIPSPARSE_LOCAL_HANDLE_HPP
|
|
#define HIPSPARSE_LOCAL_HANDLE_HPP
|
|
|
|
#include <hip/hip_runtime.h>
|
|
#include <hipsparse/hipsparse.h>
|
|
|
|
#include "hipsparse_arguments.hpp"
|
|
#include "utility.hpp"
|
|
|
|
/*! \brief local handle which is automatically created and destroyed */
|
|
class hipsparseLocalHandle_t
|
|
{
|
|
hipsparseHandle_t handle{};
|
|
|
|
public:
|
|
hipsparseLocalHandle_t()
|
|
: capture_started(false)
|
|
, graph_testing(false)
|
|
{
|
|
const hipsparseStatus_t status = hipsparseCreate(&this->handle);
|
|
if(status != HIPSPARSE_STATUS_SUCCESS)
|
|
{
|
|
throw(status);
|
|
}
|
|
}
|
|
hipsparseLocalHandle_t(const Arguments& arg)
|
|
: capture_started(false)
|
|
, graph_testing(arg.graph_test)
|
|
{
|
|
const hipsparseStatus_t status = hipsparseCreate(&this->handle);
|
|
if(status != HIPSPARSE_STATUS_SUCCESS)
|
|
{
|
|
throw(status);
|
|
}
|
|
}
|
|
~hipsparseLocalHandle_t()
|
|
{
|
|
hipsparseDestroy(this->handle);
|
|
}
|
|
|
|
// Allow hipsparseLocalHandle_t to be used anywhere hipsparseHandle_t is expected
|
|
operator hipsparseHandle_t&()
|
|
{
|
|
return this->handle;
|
|
}
|
|
operator const hipsparseHandle_t&() const
|
|
{
|
|
return this->handle;
|
|
}
|
|
|
|
void hipsparseStreamBeginCapture()
|
|
{
|
|
// As of ROCm 7.1, the HIP graph APIs require nvcc unnecessarily as they are gated by
|
|
// __CUDACC__ in the nvidia_hip_runtime_api.h header. Since we can compile hipSPARSE
|
|
// with g++/clang, these graph APIs cannot be called from hipSPARSE.
|
|
#if(!defined(CUDART_VERSION))
|
|
if(!(this->graph_testing))
|
|
{
|
|
return;
|
|
}
|
|
|
|
#ifdef GOOGLE_TEST
|
|
ASSERT_EQ(capture_started, false);
|
|
#endif
|
|
|
|
CHECK_HIP_ERROR(hipStreamCreate(&this->graph_stream));
|
|
CHECK_HIPSPARSE_ERROR(hipsparseGetStream(*this, &this->old_stream));
|
|
CHECK_HIPSPARSE_ERROR(hipsparseSetStream(*this, this->graph_stream));
|
|
|
|
// BEGIN GRAPH CAPTURE
|
|
CHECK_HIP_ERROR(hipStreamBeginCapture(this->graph_stream, hipStreamCaptureModeGlobal));
|
|
|
|
capture_started = true;
|
|
#endif
|
|
}
|
|
|
|
void hipsparseStreamEndCapture(int runs = 1)
|
|
{
|
|
// As of ROCm 7.1, the HIP graph APIs require nvcc unnecessarily as they are gated by
|
|
// __CUDACC__ in the nvidia_hip_runtime_api.h header. Since we can compile hipSPARSE
|
|
// with g++/clang, these graph APIs cannot be called from hipSPARSE.
|
|
#if(!defined(CUDART_VERSION))
|
|
if(!(this->graph_testing))
|
|
{
|
|
return;
|
|
}
|
|
|
|
#ifdef GOOGLE_TEST
|
|
ASSERT_EQ(capture_started, true);
|
|
#endif
|
|
|
|
hipGraph_t graph;
|
|
hipGraphExec_t instance;
|
|
|
|
// END GRAPH CAPTURE
|
|
CHECK_HIP_ERROR(hipStreamEndCapture(this->graph_stream, &graph));
|
|
CHECK_HIP_ERROR(hipGraphInstantiate(&instance, graph, nullptr, nullptr, 0));
|
|
|
|
CHECK_HIP_ERROR(hipGraphDestroy(graph));
|
|
CHECK_HIP_ERROR(hipGraphLaunch(instance, this->graph_stream));
|
|
CHECK_HIP_ERROR(hipStreamSynchronize(this->graph_stream));
|
|
CHECK_HIP_ERROR(hipGraphExecDestroy(instance));
|
|
|
|
CHECK_HIPSPARSE_ERROR(hipsparseSetStream(*this, this->old_stream));
|
|
CHECK_HIP_ERROR(hipStreamDestroy(this->graph_stream));
|
|
this->graph_stream = nullptr;
|
|
|
|
capture_started = false;
|
|
#endif
|
|
}
|
|
|
|
hipStream_t get_stream()
|
|
{
|
|
hipStream_t stream;
|
|
hipsparseGetStream(*this, &stream);
|
|
return stream;
|
|
}
|
|
|
|
private:
|
|
hipStream_t graph_stream;
|
|
hipStream_t old_stream;
|
|
bool capture_started;
|
|
bool graph_testing;
|
|
};
|
|
|
|
#endif // HIPSPARSE_LOCAL_HANDLE_HPP
|